function [ ] = coupling(opt, f, sigDC, hD, tVAC, dVAC)
% coupling(opt, f, sigDC, hD, tVAC, dVAC)
%
% coupling.m produces several results with loop coupling which is
% explained in LIGO-TXXXXX. 
%
% 1. Signal to noise with loop coupling
% 2. Signal to noise with loop coupling and feed forward
% 3. Signal to noise with loop coupling and dynamic range
% 4. Signal to noise with loop coupling and dynamic range and feed forward
% 5. Signal to noise with loop coupling and dynamic range and feed forward
% 6. Optical gain with/without loop coupling
% 7. Open loop gain with/without loop coupling
%
% Dynamic range limit is defined in "setup.m".
%
% Feed forward parameters is defined in "setup.m". Here 2 feed forward loops for l- to L- and ls to
% L- are used.

global rd PFf

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%%% Initial setting

  eval([char(PFf), '_setup']);

%%%%%%%%%%%% load other displacement noise from BENCH %%%%%%%%%%%%%%%%
  load bench.dat
  btotal = sqrt(bench(:, 3).^2 + bench(:, 4).^2 + bench(:, 5).^2 + bench(:, 6).^2 + bench(:, 7).^2) * 4000;
  btotalf = interp1(bench(:,1), btotal*0, f);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%%% Coupling


  %%% display UGF
  for pn = 1 : length(PN)
    fprintf('Unity gain frequency for %s : %gHz\n', char(NC(pn)), UGF(pn));
  end
  fprintf('\n');

  %%% Dynamic range
  for pn = 1 : length(PN)
    DrangeDEF(pn) = VdcMAX(pn) / PDnoise(pn);
    if isempty(findstr('IM', opt.probe(PN(pn)).name))
      if isempty(findstr('DC', opt.probe(PN(pn)).name))
        %for single demodulation         
        if isempty(findstr('Q', opt.probe(PN(pn)).name))
          Drange(pn) = abs(sqrt((sigDC(PN(pn), 1))^2 + sigDC(PN(pn)+1, 1)^2) / min(dVAC(:, PN(pn)))); %%% take max offset
        else
          Drange(pn) = abs(sqrt((sigDC(PN(pn)-1, 1))^2 + sigDC(PN(pn), 1)^2) / min(dVAC(:, PN(pn)))); %%% take max offset
        end
        else
        %for DC readout         
        Drange(pn) = abs(sigDC(PN(pn), 1) / min(tVAC(:, pn)));
      end
      Dcomp(pn) = sqrt(1 + (Drange(pn) / DrangeDEF(pn))^2);
    else
      %for DDM, it takes maximum offset when dmodulation phases are swept.         
      DrangeM = abs(sqrt((sigDC(PN(pn), 1))^2 + sigDC(PN(pn)+1, 1)^2) / min(dVAC(:, PN(pn))));
      DrangeP = abs(sqrt((sigDC(PN(pn)+2, 1))^2 + sigDC(PN(pn)+3, 1)^2) / min(dVAC(:, PN(pn)+2)));
      Dcomp(pn) = sqrt(1 + ((DrangeM / DrangeDEF(pn) + DrangeP / DrangeDEF(pn)) / 2 )^2);
    end
    fprintf('Dynamic range limit factor for %s : %g with DC offset by %gV and PD noise by %gV/rtHz\n',...
          char(ND(pn)), Dcomp(pn), VdcMAX(pn), PDnoise(pn));
  end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%%% Main

% Gij   : servo-gain matrix
% Tij   : feed-forward matrix
% hD_ij : optical-gain matrix
% x_i  : displacement-noise vector, all 0 here
% tVAC_i : quantum noise vector
% 
% According to eq.(2) of matrix.pdf, we shall define
% 
% D_{ij} = hD_{ij}
%
% and then we have
%
% y_i = inv(1-M_{ij}) * (D_{ij} x_j + n_i)

  
for FFloop = 1 : 2
  FF = zeros(length(PN), length(PN));

  if FFloop == 2
    FF(Lm, lp) = FF_Lm2lp;
    FF(Lm, lm) = FF_Lm2lm;
    FF(Lm, lsr) = FF_Lm2lsr;
  end

  Gij = zeros(length(f), length(PN), length(PN));
  Tij = zeros(length(f), length(PN), length(PN));
  xi = zeros(length(PN), length(f));
  Id = eye(length(PN));

  for ff = 1 : length(f)
    clear Mij Dij Bij BD
    Mij = zeros(length(PN), length(PN));
    %%% No coupling case
    %ht(:, :) = hD(ff, :, :);
    %hD(ff, :, :) = ht .* Id;
    %%% 

    for a = 1 : length(PN)
      for b = 1 : length(PN)
        Tij(ff, a, b) = FF(a,b);
      end
      Gij(ff, a, a) = UGF(a) / f(ff) / hD(ff, a, a);
      OLe(ff, a) = Gij(ff, a, a) * hD(ff, a, a); % expected OLTF
    end

    for a = 1 : length(PN)
      for b = 1 : length(PN)
        % Coupling of hD(f, a, b) is redued by FF.
        % Second hD(ff, a, b) is a special case to reduce coupling in all frequency.
        % In general, hD(ff, a, b) should be Cij(ff, a, b) * (hD(ff, a, a); Cij is
        % a compensative filter to match the shape beween b and a and it has frequemcy dependence.
        Mij(a, b) = - Gij(ff, b, b) * hD(ff, a, b) - Tij(ff, a, b) * Gij(ff, b, b) * hD(ff, a, b);
      end
    end
    Dij(:, :) = hD(ff, :, :);  % Dij at one particular frequency
    Bij = inv(Id - Mij);       % Bij = (1 - Mij)^-1
    BD = Bij * Dij;
    for a = 1 : length(PN)
      CL(ff, a) = Gij(ff, a, a) * BD(a, a); % closed loop TF
      OL(ff, a) = CL(ff, a) / (CL(ff, a) - 1); % effective open loop TF
      EOTF(ff, a) = OL(ff, a) / Gij(ff, a, a); % effctive optical TF

      for b = 1 : length(PN)
        yt(ff, a, b) = sqrt((BD(a, b) * xi(b, ff))^2) + sqrt((Bij(a, b) * tVAC(ff, b))^2); % each noise contribution to y_i
        hGWt(ff, a, b) = yt(ff, a, b) / BD(a, a); % each signal to noise ratio

        % including dynamic range limit
        ytD(ff, a, b) = sqrt((BD(a, b) * xi(b, ff))^2) + sqrt((Bij(a, b) * (tVAC(ff, b) * Dcomp(b)))^2); % each noise contribution to y_i
        hGWtD(ff, a, b) = ytD(ff, a, b) / BD(a, a); % each of signal to noise ratio
      end
      yi(ff, a) = 0;
      for b = 1 : length(PN)
%        yi(ff, a) = sqrt(yt(ff, a, 1)^2 + yt(ff, a, 2)^2 + yt(ff, a, 3)^2 +yt(ff, a, 4)^2 +yt(ff, a, 5)^2); %total y_i
        yi(ff, a) = sqrt(yi(ff, a)^2 + yt(ff, a, b)^2); %total y_i
      end
      hGWa(ff, a) = yi(ff, a) / BD(a, a); % signal to noise from total y_i

      % including dynamic range limit
      yiD(ff, a) = 0;
      for b = 1 : length(PN)
%      yiD(ff, a) = sqrt(abs(ytD(ff, a, 1))^2 + abs(ytD(ff, a, 2))^2 + abs(ytD(ff, a, 3))^2 + abs(ytD(ff, a, 4))^2 +abs(ytD(ff, a, 5))^2); %total y_i
        yiD(ff, a) = sqrt(yiD(ff, a)^2 + abs(ytD(ff, a, b))^2); %total y_i
      end
      hGWaD(ff, a) = yiD(ff, a) / BD(a, a); % signal to noise from total y_i
    end
  end

  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
  %%% Plots

  figure(600 + (FFloop-1))
  clf
%  loglog(f, tVAC(:, Lm) ./ abs(hD(:, Lm, Lm)), ':k', 'LineWidth', 2)
%  loglog(f, [abs(hGWtD(:, 2, Lp)), abs(hGWtD(:, 2, Lm)), abs(hGWtD(:, 2, lp)), abs(hGWtD(:, 2, lm)), abs(hGWtD(:, 2, lsr))], 'Linewidth',2)
  hGWtDg = zeros(length(PN), length(f));
  for b = 1 : length(PN)
    hGWtDg(b, :) = abs(hGWtD(:, 2, b));
  end
  loglog(f, hGWtDg, 'Linewidth',2)
  hold on
  loglog(f, abs(hGWaD(:, Lm)), 'k', 'LineWidth', 2)
  loglog(f, sqrt(abs(hGWaD(:, Lm)).^2 + btotalf.^2), ':k', 'LineWidth', 3)
  %loglog(bench(:,1), [bench(:,3), bench(:,4), bench(:,5), bench(:,6), bench(:,7)]*4000, ':c', 'Linewidth',2);
  clear lg
  for b = 1 : length(PN)
    lg(b) = {strcat(char(NC(b)), ' at ',  char(ND(b)))};
  end
  lg(b+1) = {'quantum total'};
  lg(b+2) = {'total'};
  %lg(b+3) = {'other displacement'};
  legend(lg)
  xlabel('Hz')
  ylabel('m/rHz')
  axis([1 10000 1e-21 1e-15])
  grid on
  hold off
  if FFloop == 1
    title(['Signal to noise with loop coupling with no feed forward'])
    h = gcf;
    for ns = 1 : length(ext)
      saveas(h, [rd '/' num2str(600 + (FFloop-1)) '_loopnoise.' char(ext(ns))]);
   end
  else
    title(['Signal to noise with loop coupling and feed forward'])
    h = gcf;
    for ns = 1 : length(ext)
      saveas(h, [rd '/' num2str(600 + (FFloop-1)) '_loopnoiseFF.' char(ext(ns))]);
    end
  end

  %figure(602 + (FFloop-1))
  %clf
  %%loglog(f, tVAC(:, Lm) ./ abs(hD(:, Lm, Lm)), ':k', 'LineWidth', 2)
  %loglog(f, [abs(hGWt(:, 2, Lp)), abs(hGWt(:, 2, Lm)), abs(hGWt(:, 2, lp)), abs(hGWt(:, 2, lm)), abs(hGWt(:, 2, lsr))], 'Linewidth',2)
  %hold on
  %loglog(f, abs(hGWa(:, Lm)), 'k', 'LineWidth', 2)
  %loglog(f, sqrt(abs(hGWaD(:, Lm)).^2 + btotalf.^2), ':k', 'LineWidth', 3)
  %loglog(bench(:,1), [bench(:,3), bench(:,4), bench(:,5), bench(:,6), bench(:,7)]*4000, ':c', 'Linewidth',2);
  %for b = 1 : length(PN)
  %  lg(b) = {strcat(char(NC(b)), ' at ',  char(ND(b)))};
  %end
  %lg(b+1) = {'quantum total'};
  %lg(b+2) = {'total'};
  %lg(b+3) = {'other displacement'};
  %legend(lg)  %xlabel('Hz')
  %ylabel('m/rHz')
  %axis([5 7000 1e-22 1e-16])
  %grid on
  %hold off
  %if FFloop == 1
  %  title('Signal to noise with loop coupling')
  %  h = gcf;
  %  saveas(h, [rd '/' num2str(602 + (FFloop-1)) '_loopnoise_noDrange.' ext]);
  %else
  %  title('Signal to noise with loop coupling and feed forward')
  %  h = gcf;
  %  saveas(h, [rd '/' num2str(602 + (FFloop-1)) '_loopnoiseFF_noDrange.' ext]);
  %end
  
end

  figure(610)
  hDg = zeros(length(PN), length(f));
  for b = 1 : length(PN)
    hDg(b, :) = hD(:, b, b);
  end
  loglog(f, hDg, ':', 'LineWidth', 2)
  %loglog(f, [abs(hD(:, 1, 1)), abs(hD(:, 2, 2)), abs(hD(:, 3, 3)), abs(hD(:, 4, 4)), abs(hD(:, 5, 5))], ':', 'LineWidth', 2)
  hold on
  loglog(f, abs(EOTF(:, :)))
  title('Optical gain: dotted; no coupling, solid; with coupling')
  clear lg
  for b = 1 : length(PN)
    lg(b) = {strcat(char(NC(b)), ' at ',  char(ND(b)))};
  end
  legend(lg)
  xlabel('Hz')
  ylabel('W/m rHz')
  grid on
  hold off
  h = gcf;
  for ns = 1 : length(ext)
    saveas(h, [rd '/' num2str(610) '_loopoptgain.' char(ext(ns))]);
  end
    
  figure(611)
  loglog(f, abs(OL(:, :)))
  hold on
  loglog(f, abs(OLe(:, :)), ':', 'LineWidth', 2)
  legend('L+ loop', 'L- loop', 'l+ loop', 'l- loop', 'ls loop')
  clear lg
  for b = 1 : length(PN)
    lg(b) = {strcat(char(NC(b)), ' loop')};
  end
  legend(lg)
  title('OLTF: dotted; no coupling, solid; with coupling')
  xlabel('Hz')
  ylabel('Gain')
  grid on
  hold off
  h = gcf;
  for ns = 1 : length(ext)
    saveas(h, [rd '/' num2str(611) '_loopOLTF.' char(ext(ns))]);
  end
  
  %figure(612)
  %loglog(f, abs(CL(:, :)))
  %legend('L+ loop', 'L- loop', 'l+ loop', 'l- loop', 'ls loop')
  %title('Cloed loop TF with coupling')
  %xlabel('Hz')
  %ylabel('Gain')
  %grid on
  %h = gcf;
  %saveas(h, [rd '/' num2str(612) '_loopCLTF.' ext]);

  %figure(613)
  %loglog(f, abs(Gj(:, :)))
  %legend('L+', 'L-', 'l+', 'l-', 'ls')
  %title('Feedback filter TF designed by original optical gain H_ii')
  %xlabel('Hz')
  %ylabel('Gain')
  %grid on
  %h = gcf;
  %saveas(h, [rd '/' num2str(613) '_filter.' ext]);
