% Create an Optickle model for LCGT in 2009/06
%
%
% lPRC = lPR + (lIX + lIY) / 2;
% lSRC = lSR + (lIX + lIY) / 2;
% lasy = lIX - lIY;
%
% dTune - the SRCL detuning distance
%
% The detuning phase used in the Bench is the round trip phase:
%   ifo.Optics.SRM.Tunephase = 2 * k * dTune
% where k = 2 * pi / lambda
%
% Buonanno & Chen gr-qc/0102012 use the one-way phase, given by:
%   Chen.Tunephase = pi / 2 - k * dTune
%  which is similar to the total offset on the SR set with:
%   opt = setPosOffset(opt, nSR, opt.lambda / 4 + dTune);
%  since
%   Chen.Tunephase = k * (lambda / 4 + dTune)
%
%  The lambda / 4 is there to get close to the resonance, and 
%  dTune to set the detuning frequency.  For the LIGO2 detector,
%  the detuning frequency is roughly given by:
%    detuneFreq =?= tan(k * (lambda / 4 + dTune)) * (-13 Hz)
%

function opt = optLCGT_DRSE

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Detector Parameters

  %%% detector geometry (distances in meters)
  lEX = 3006.69;        % length of the X arm
  lEY = 3006.69;        % length of the Y arm
  lass = 6.66205;       % asymmetry length
  lPR = 54.96195 - 20;  % distance from PR to BS
  lSR = 64.95503 - 20;  % distance from SR to BS
  lIX = 20 + lass/2;    % distance from BS to IX
  lIY = 20 - lass/2;    % distance from BS to IY

%  dTune = (12)/360*1064e-9;     % SR micro detuning (see note) for NS/NS, 2.5degree 
%  dTune = (11.25)/360*1064e-9;     % SR micro detuning (see note) for NS/NS, 2.5degree 
  dTune = (13.5)/360*1064e-9;     % SR micro detuning (see note) for NS/NS, 2.5degree 
  Eoffset = 1e-12;  % offset on ETM for DC readout
  %Eoffset = 150e-12;  % offset on ETM for DC readout
  
  %%% optical parameters
  %
  
  % Typical loss at test mass
  Loss = 45e-6;             % Loss at HR surface
  BSLoss = 100e-6;          % Loss at HR surface for BS
  ARLoss = 1000e-6;         % Loss at AR surface
  bulklossL = 20e-6 * 0.15; % Loss inside substrate for large optics
  bulklossS = 2.5e-6 * 0.1; % Loss inside substrate for small optics
  CLoss = 750e-6*0;           % as a contrast defect
  
  % Reflectivity of test masses
  Retm = 0.99995;
  Ritm = 0.992;
  Rprm = 0.77;
  Rsrm = 0.94^2;

  Tetm = 1 - Retm - Loss;
  Titm = 1 - Ritm - Loss;
  Tprm = 1 - Rprm - Loss;
  Tsrm = 1 - Rsrm - Loss;

  % ITM inbalance
  dTitm = Titm/100;
  
  % the loss on EX and EY is used to set the contrast defect
  dLoss = 5e-6;

  % having 1% reflection at REFL
  dTprm = 0; 

  % transmissivity of AR surface
  TAR = 0.999;
  
  % RF pickoff for MICH, default:0.9 which means 10% goes to RF port
  RFPO_OMCR = 1;
  DC = 1; % 1:DC readout, 0:RF readout. It changes T_OMCs.

  % Attenuator setting placed in front of PD to adjust PD input power
  Att_REFL  = 0.006;
  Att_REFLb = 0.006;
  Att_AS    = 1;
  Att_OMC   = 1;
  Att_OMCRb = 1;
  Att_POX   = 0.25;
  Att_POXb  = 0.25;
  Att_POY   = 1;
  Att_POM   = 0.13;
  
  % radius of curvature [m]
  Ri = 10^10;   % input mirrors (IX and IY), not used
  Re = 4500;	% end mirrors (EX and EY), not used
  Rr = 0.81;	% recycling mirrors (PR and SR), not used
  
  
  % mechanical parameters
  wI = 2 * pi * 0.8;		% resonance frequency of input mirror (rad/s)
  mI = 30;                  % mass of input mirror (kg)
  wE = 2 * pi * 0.8;		% resonance frequency of end mirror (rad/s)
  mE = 30;                  % mass of end mirror (kg)
  
  % input beam parameters
%  Pin = 83.982;		% input power (W), carrier power will be 75W.
%  Pin = 75.755;		% input power (W), carrier power will be 75W.
  Pin = 75;		% input power (W), carrier power will be 75W.
%  f1 = 11.25e6;     % first modulation frequency
  f1 = 16.3636e6;     % first modulation frequency
  f2 = 45e6;		% second modulation frequency
  mod1 = 'pm';      % modulation method for f1 : 'am' or 'pm'
  mod2 = 'pm';      % modulation method for f2 : 'am' or 'pm'
  g1 = 0.2;			% first modulation depth; effectively g = 0.1 if no MZ 
  g2 = 0.2;			% second modulation depth; effectively g = 0.1 if no MZ

  
  
  
  
  
  
  
  
  Nmod1 = 5;		% first modulation order
  Nmod2 = 5;		% second modulation order
  
  
  
  
  
  
  
  if 1
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % construct modulation vectors (Mach Z) 
  if strcmp(mod1, 'am')
    Nmod1 = 1;		% first modulation order
  end
  if strcmp(mod2, 'am')
    Nmod2 = 1;		% second modulation order
  end
  n1 = (-Nmod1:Nmod1)';
  n2 = (-Nmod2:Nmod2)';
  N1 = length(n1);
  N2 = length(n2);

  vMod1 = zeros(N1, 2);        % f1 sideband vector
  for n = 1:N1
    ff = n1(n) * f1;
    if strcmp(mod1, 'am') % passive amplitude modulators (like acousto-
                          % optic modulators or electro-optic modulators
                          % with polarisers) can only reduce the amplitude.
      if n1(n) == 0
        aa = sqrt(Pin/2) * (1 - g1/2);  % amplitude modulation for CA, see FINESSE0.93 manual
      else
        aa = sqrt(Pin/2) * g1/4; % amplitude modulation for SB, see FINESSE0.93 manual
      end
    else
      aa = sqrt(Pin/2) * besselj(n1(n), g1); % phase modulation
    end
    vMod1(n,:) = [ff, aa/sqrt(2)];   % sqrt(2) for the MZ output mirror
  end
 
  vMod2 = zeros(N2, 2);        % f2 sideband vector
  for n = 1:N2
    ff = n2(n) * f2;
    if strcmp(mod2, 'am') % passive amplitude modulators (like acousto-
                          % optic modulators or electro-optic modulators
                          % with polarisers) can only reduce the amplitude.
      if n2(n) == 0
        aa = sqrt(Pin/2) * (1 - g2/2); % amplitude modulation for CA
      else
        aa = sqrt(Pin/2) * g2/4; % amplitude modulation for SB
      end
    else
      aa = sqrt(Pin/2) * besselj(n2(n), g2);  % phase modulation
    end
    vMod2(n,:) = [ff, aa/sqrt(2)];   % sqrt(2) for the MZ output mirror
  end

  % add them together constructively
  vMod = vMod1;
  for n = 1:N2
    % if the N2 frequency is already in the list, add to the exisiting field
    if sum(ismember(vMod(:, 1),vMod2(n, 1)))
      gg = find(vMod(:, 1) == vMod2(n, 1));
      vMod(gg, 2) = vMod(gg, 2) + vMod2(n, 2);  % constructive interference
    % else add the new frequency and field to the list of modulations
    else
      vMod = [vMod;vMod2(n,:)];
    end
  end
  vMod = sortrows(vMod, 1);
  aMax = max(abs(vMod(:, 2)));
  end

  
  
if 0
  n1 = (-Nmod1:Nmod1)';
  n2 = (-Nmod2:Nmod2)';
  N1 = length(n1);
  N2 = length(n2);
  vMod = zeros(N1 * N2, 2);
  for n = 1:N1
    nn = N1 * (n - 1) + (1:N2);
    ff = n1(n) * f1 + n2 * f2;
    aa = sqrt(Pin) * bessel(n1(n), g1) * bessel(n2, g2);
    vMod(nn, :) = [ff, aa];
  end
  vMod = sortrows(vMod, 1);
  aMax = max(abs(vMod(:, 2)));
end  
  
  
  % create an empty model
  opt = Optickle;

  
  
      
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Add a Field Source

  % make the modulation vectors and add a source
  [opt, nSrc] = addSource(opt, 'Source', vMod);
  
  % add an AM modulator
  [opt, nAM] = addModulator(opt, 'AM', 1i);


  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Add Mirrors
  % The argument list for addMirror is:
  % [opt, sn] = addMirror(opt, name, aio, Chr, Thr, Lhr, Rar)
  % type "help addMirror" for more information
  
  % add recycling mirrors
  [opt, nPR] = addMirror(opt, 'PR', 0, 1 / Rr, Tprm + dTprm, Loss, ARLoss, bulklossS);
  %[opt, nPR] = addMirror(opt, 'PR', 0, 1 / Rr, 1, 0, 0);
  %[opt, nBSPM] = addBeamSplitter(opt, 'BSPM', 45, 0, TAR*0+1, Loss*0, ARLoss*0, bulklossS*0);
  [opt, nBSPM] = addBeamSplitter(opt, 'BSPM', 45, 0, TAR, Loss, ARLoss, bulklossS);

  [opt, nSR] = addMirror(opt, 'SR', 0, 1 / Rr, Tsrm, Loss, ARLoss, bulklossS);
  %[opt, nSR] = addMirror(opt, 'SR', 0, 1 / Rr, 1, 0, 0);

  % add beam-splitter
  [opt, nBS] = addBeamSplitter(opt, 'BS', 45, 0, 0.5 - BSLoss/2, BSLoss, ARLoss*1, bulklossS);
  
  % add arm mirrors (a.k.a. test-masses)
  [opt, nBSPX] = addBeamSplitter(opt, 'BSPX', 45, 0, TAR, Loss*0, ARLoss*0 + CLoss*0, 0);
  [opt, nBSPY] = addBeamSplitter(opt, 'BSPY', 45, 0, TAR, Loss*0, ARLoss*0 + CLoss*0, 0);
  [opt, nIX] = addMirror(opt, 'IX', 0, 1 / Ri, Titm + dTitm, Loss, ARLoss, bulklossL);
  [opt, nIY] = addMirror(opt, 'IY', 0, 1 / Ri, Titm - dTitm, Loss, ARLoss, bulklossL);

  [opt, nEX] = addMirror(opt, 'EX', 0, 1 / Re, Tetm - dLoss, Loss + dLoss, ARLoss, bulklossL);
  [opt, nEY] = addMirror(opt, 'EY', 0, 1 / Re, Tetm - dLoss, Loss - dLoss, ARLoss, bulklossL);
  %[opt, nEX] = addMirror(opt, 'EX', 0, 1 / Re, 1 , 0, 0); %no ETM
  %[opt, nEY] = addMirror(opt, 'EY', 0, 1 / Re, 1 , 0, 0); %no ETM
  
  % OMC mirrors
  [opt, nOMCa] = addMirror(opt, 'OMCa', 0, 1, pi/500 * DC, 0, 0); % T=1 for no OMC, T=0 for RF detection at OMC and OMCRb
  [opt, nOMCb] = addMirror(opt, 'OMCb', 0, 1, pi/500, 0, 0); % T=1 for no OMC

  % Homodyne detector
  % pick off for HD
  [opt, nPOHD] = addBeamSplitter(opt, 'POHD', 45, 0, 1, 0, 0); %0% pick off
  [opt, nFL] = addBeamSplitter(opt, 'FL', 45, 0, 0, 0, 0); % flipper mirror: T=1 for HD, T=0 for AS
  [opt, nHDBS] = addBeamSplitter(opt, 'HDBS', 45, 0, 0.5, 0, 0);

  % set mechanical transfer-functions for the test-masses (m/N)
%  dampedRes = [-0.1 + 1i, -0.1 - 1i];
  dampedRes = [-0.12 + 1i, -0.12 - 1i];

  opt = setMechTF(opt, nIX, zpk([], wI * dampedRes, 2 / mI));
  opt = setMechTF(opt, nEX, zpk([], wE * dampedRes, 2 / mE));
  opt = setMechTF(opt, nIY, zpk([], wI * dampedRes, 2 / mI));
  opt = setMechTF(opt, nEY, zpk([], wE * dampedRes, 2 / mE));

  % set position offsets for SR
  opt = setPosOffset(opt, nSR, opt.lambda / 4 + dTune);
  
  % set position offsets for DARM for DCreadout producing 100mW at dark port
  % Homodyne phase corresponds ?? degree with loss difference at ETMs above
  opt = setPosOffset(opt, nEX, Eoffset);
  opt = setPosOffset(opt, nEY, -1*Eoffset);
  %opt = setPosOffset(opt, nHDBS, -2.5e-10);
  
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Add attenuators (to avoid saturation)
  [opt, nAttREFL] = addBeamSplitter(opt, 'AttREFL', 45, 0, Att_REFL, 0, 0);
  [opt, nAttREFLb] = addBeamSplitter(opt, 'AttREFLb', 45, 0, Att_REFLb, 0, 0);
  [opt, nAttAS]   = addBeamSplitter(opt, 'AttAS',   45, 0, Att_AS, 0, 0);
  [opt, nAttOMC]  = addBeamSplitter(opt, 'AttOMC',  45, 0, Att_OMC, 0, 0);
  [opt, nAttOMCRb]  = addBeamSplitter(opt, 'AttOMCRb',  45, 0, Att_OMCRb, 0, 0);
  [opt, nAttPOX]  = addBeamSplitter(opt, 'AttPOX',  45, 0, Att_POX, 0, 0);
  [opt, nAttPOXb]  = addBeamSplitter(opt, 'AttPOXb',  45, 0, Att_POXb, 0, 0);
  [opt, nAttPOY]  = addBeamSplitter(opt, 'AttPOY',  45, 0, Att_POY, 0, 0);
  [opt, nAttPOM]  = addBeamSplitter(opt, 'AttPOM',  45, 0, Att_POM, 0, 0);
  %[opt, nAttHDa]  = addBeamSplitter(opt, 'AttHDa',  45, 0, 1, 0, 0); % dummy
  %[opt, nAttHDb]  = addBeamSplitter(opt, 'AttHDb',  45, 0, 1, 0, 0); % dummy


  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Add Field Sinks (needed by the probes)
  [opt, nBSREFLa] = addBeamSplitter(opt, 'BSREFLa', 45, 0, 0.5, 0, 0); % put T=0.5 if 2 ports are needed at REFL
  [opt, nREFL] = addSink(opt, 'REFL');
  [opt, nREFLb] = addSink(opt, 'REFLb');
  [opt, nAS] = addSink(opt, 'AS');
  [opt, nBSPOXa] = addBeamSplitter(opt, 'BSPOXa', 45, 0, 1, 0, 0);
  [opt, nPOX] = addSink(opt, 'POX');
  [opt, nPOXb] = addSink(opt, 'POXb');
  [opt, nPOY] = addSink(opt, 'POY');
  [opt, nPOM] = addSink(opt, 'POM');
  [opt, nTRX] = addSink(opt, 'TRX');
  [opt, nTRY] = addSink(opt, 'TRY');
  [opt, nBSOMCRa] = addBeamSplitter(opt, 'BSOMCRa', 45, 0, RFPO_OMCR, 0, 0); % put 0.9 for 10% pick off for MICH
  [opt, nOMC] = addSink(opt, 'OMC');
  [opt, nOMCRb] = addSink(opt, 'OMCRb');
  [opt, nHDa] = addSink(opt, 'HDa');
  [opt, nHDb] = addSink(opt, 'HDb');

  
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %%% add vacuums

  % pick off for vacuum
  Pnor = 1e-5 * aMax^2;
  [opt, nPOV] = addBeamSplitter(opt, 'POV', 45, 0, 1 - 1e-5, 0, 0); %0.001% pick off

  %MC for vacuum to avoid RF modulation
  [opt, nVMCa] = addMirror(opt, 'VMCa', 0, 1, pi/500, 0, 0);
  [opt, nVMCb] = addMirror(opt, 'VMCb', 0, 1, pi/500, 0, 0);

  % attenuator to adjust vacuum amplitude to 9.3348e-020W for 10 vacuums
  [opt, nATT] = addMirror(opt, 'ATT', 0, 1, 1.8670e-19/2 / Pnor * 10, 0, 0);
  
  % splitters for each vacuum
  [opt, nPOAS] = addBeamSplitter(opt, 'POAS', 45, 0, 1 - 1/10, 0, 0);
  [opt, nPOIXARi] = addBeamSplitter(opt, 'POIXARi', 45, 0, 1 - 1/9, 0, 0);
  [opt, nPOIXARo] = addBeamSplitter(opt, 'POIXARo', 45, 0, 1 - 1/8, 0, 0);
  [opt, nPOIYARi] = addBeamSplitter(opt, 'POIYARi', 45, 0, 1 - 1/7, 0, 0);
  [opt, nPOIYARo] = addBeamSplitter(opt, 'POIYARo', 45, 0, 1 - 1/6 , 0, 0);
  [opt, nPOAttREFL] = addBeamSplitter(opt, 'POAttREFL', 45, 0, 1 - 1/5, 0, 0);
  [opt, nPOAttAS] = addBeamSplitter(opt, 'POAttAS', 45, 0, 1 - 1/4, 0, 0);
  [opt, nPOAttOMC] = addBeamSplitter(opt, 'POAttOMC', 45, 0, 1 - 1/3, 0, 0);
  [opt, nPOAttPOX] = addBeamSplitter(opt, 'POAttPOX', 45, 0, 1 - 1/2, 0, 0);

  
  %%% add vacuum for 2-photon modes (1 and i)

  % for laser
  [opt, nVREI] = addModulator(opt, 'VREI', 1 * sqrt(1.8670e-19 / 2 / Pin));
  [opt, nVREQ] = addModulator(opt, 'VREQ', 1i * sqrt(1.8670e-19 / 2 / Pin));
  % for dark port
  [opt, nVASI] = addModulator(opt, 'VASI', 1);
  [opt, nVASQ] = addModulator(opt, 'VASQ', 1i);
  % for POX AR
  [opt, nVIXARiI] = addModulator(opt, 'VIXARiI', 1);
  [opt, nVIXARiQ] = addModulator(opt, 'VIXARiQ', 1i);
  [opt, nVIXARoI] = addModulator(opt, 'VIXARoI', 11);
  [opt, nVIXARoQ] = addModulator(opt, 'VIXARoQ', 1i);
  % for POY AR
  [opt, nVIYARiI] = addModulator(opt, 'VIYARiI', 1);
  [opt, nVIYARiQ] = addModulator(opt, 'VIYARiQ', 1i);
  [opt, nVIYARoI] = addModulator(opt, 'VIYARoI', 1);
  [opt, nVIYARoQ] = addModulator(opt, 'VIYARoQ', 1i);
  % to REFL PDs
  [opt, nVAttREFLI] = addModulator(opt, 'VAttREFLI', 1);
  [opt, nVAttREFLQ] = addModulator(opt, 'VAttREFLQ', 1i);
  % to AS PD
  [opt, nVAttASI] = addModulator(opt, 'VAttASI', 1);
  [opt, nVAttASQ] = addModulator(opt, 'VAttASQ', 1i);
  % to OMCR PD
  [opt, nVAttOMCI] = addModulator(opt, 'VAttOMCI', 1);
  [opt, nVAttOMCQ] = addModulator(opt, 'VAttOMCQ', 1i);
  % to POX PD
  [opt, nVAttPOXI] = addModulator(opt, 'VAttPOXI', 1);
  [opt, nVAttPOXQ] = addModulator(opt, 'VAttPOXQ', 1i);

  
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Add Links

  % link input to PR through PO for vacuum and PO for HD
  opt = addLink(opt, nSrc, 'out', nPOV, 'frA', 1);
  opt = addLink(opt, nPOV, 'bkA', nPOHD, 'frA', 1);
  opt = addLink(opt, nPOHD, 'bkA', nAM, 'in', 1);

  % link BS A-side inputs to PR and SR front outputs
  opt = addLink(opt, nPR, 'fr', nBSPM, 'frA', 0);
  opt = addLink(opt, nBSPM, 'bkA', nBS, 'frA', lPR);
  opt = addLink(opt, nSR, 'fr', nBS, 'bkA', lSR);

  % link BS A-side outputs to and IX and IY back inputs
  opt = addLink(opt, nBS, 'frA', nBSPY, 'frA', lIY); % for vacuum injection on POY
  opt = addLink(opt, nBS, 'bkA', nBSPX, 'frA', lIX); % for vacuum injection on POX
  opt = addLink(opt, nBSPY, 'bkA', nIY, 'bk', 0);
  opt = addLink(opt, nBSPX, 'bkA', nIX, 'bk', 0);

  % link BS B-side inputs to and IX and IY back outputs
  opt = addLink(opt, nIY, 'bk', nBSPY, 'bkB', 0);
  opt = addLink(opt, nIX, 'bk', nBSPX, 'bkB', 0);
  opt = addLink(opt, nBSPY, 'frB', nBS, 'frB', lIY);
  opt = addLink(opt, nBSPX, 'frB', nBS, 'bkB', lIX);

  % link BS B-side outputs to PR and SR front inputs
  opt = addLink(opt, nBS, 'frB', nBSPM, 'bkB', lPR);
  opt = addLink(opt, nBSPM, 'frB', nPR, 'fr', 0);
  opt = addLink(opt, nBS, 'bkB', nSR, 'fr', lSR);

  % link the arms
  opt = addLink(opt, nIX, 'fr', nEX, 'fr', lEX);
  opt = addLink(opt, nEX, 'fr', nIX, 'fr', lEX);

  opt = addLink(opt, nIY, 'fr', nEY, 'fr', lEY);
  opt = addLink(opt, nEY, 'fr', nIY, 'fr', lEY);

  % link the OMC
  opt = addLink(opt, nSR, 'bk', nOMCa, 'bk', 5);
  opt = addLink(opt, nOMCa, 'fr', nOMCb, 'fr', 0.22);
  opt = addLink(opt, nOMCb, 'fr', nOMCa, 'fr', 0.22);

  % link the HD
  opt = addLink(opt, nPOHD, 'frA', nHDBS, 'frA', 1);
  opt = addLink(opt, nOMCb, 'bk', nFL, 'frA', 1);
  opt = addLink(opt, nFL, 'bkA', nHDBS, 'bkA', 1);
  opt = addLink(opt, nHDBS, 'bkA', nHDa, 'in', 1);
  opt = addLink(opt, nHDBS, 'frA', nHDb, 'in', 1);

  
  % link attenuators and the detectors

  %opt = addLink(opt, nPR, 'bk', nAttREFL, 'frA', 5);
  %opt = addLink(opt, nAttREFL, 'bkA', nBSREFLa, 'frA', 1);
  %opt = addLink(opt, nBSREFLa, 'bkA', nREFL, 'in', 1);
  %opt = addLink(opt, nBSREFLa, 'frA', nREFLb, 'in', 1);

  opt = addLink(opt, nPR, 'bk', nBSREFLa, 'frA', 5);
  opt = addLink(opt, nBSREFLa, 'bkA', nAttREFL, 'frA', 1);
  opt = addLink(opt, nBSREFLa, 'frA', nAttREFLb, 'frA', 1);
  opt = addLink(opt, nAttREFL, 'bkA', nREFL, 'in', 1);
  opt = addLink(opt, nAttREFLb, 'bkA', nREFLb, 'in', 1);


  opt = addLink(opt, nBSPX, 'frA', nBSPOXa, 'frA', 1);
  opt = addLink(opt, nBSPOXa, 'bkA', nAttPOX, 'frA', 1);
  opt = addLink(opt, nAttPOX, 'bkA', nPOX, 'in', 1);
  opt = addLink(opt, nBSPOXa, 'frA', nAttPOXb, 'frA', 1);
  opt = addLink(opt, nAttPOXb, 'bkA', nPOXb, 'in', 1);  
  
  opt = addLink(opt, nBSPY, 'frA', nAttPOY, 'frA', 5);
  opt = addLink(opt, nAttPOY, 'bkA', nPOY, 'in', 5);

  opt = addLink(opt, nBSPM, 'frA', nAttPOM, 'frA', 5);
  opt = addLink(opt, nAttPOM, 'bkA', nPOM, 'in', 1);

  opt = addLink(opt, nEX, 'bk', nTRX, 'in', 5);
  opt = addLink(opt, nEY, 'bk', nTRY, 'in', 5);


  
  opt = addLink(opt, nFL, 'frA', nAttAS, 'frA', 5);
  opt = addLink(opt, nAttAS, 'bkA', nAS, 'in', 1);

  opt = addLink(opt, nOMCa, 'bk', nBSOMCRa, 'frA', 1);
  opt = addLink(opt, nBSOMCRa, 'bkA', nAttOMC, 'frA', 1);
  opt = addLink(opt, nBSOMCRa, 'frA', nAttOMCRb, 'frA', 1);
  opt = addLink(opt, nAttOMC, 'bkA', nOMC, 'in', 1);
  opt = addLink(opt, nAttOMCRb, 'bkA', nOMCRb, 'in', 1);

  
  %%% link vacuum

  % vcuum for laser
  opt = addLink(opt, nAM, 'out', nVREI, 'in', 10);
  opt = addLink(opt, nVREI, 'out', nVREQ, 'in', 0);
  opt = addLink(opt, nVREQ, 'out', nPR, 'bk', 1);

  %MC for vacuum
  opt = addLink(opt, nPOV, 'frA', nVMCa, 'bk', 1);
  opt = addLink(opt, nVMCa, 'fr', nVMCb, 'fr', 0.22);
  opt = addLink(opt, nVMCb, 'fr', nVMCa, 'fr', 0.22);
  opt = addLink(opt, nVMCb, 'bk', nATT, 'bk', 1);

  
  %%% attenuator to adjust vacuum amplitude to 9.3348e-020W for 10 vacuums

  opt = addLink(opt, nATT, 'fr', nPOAS, 'frA', 1);

  % for dark port
  opt = addLink(opt, nPOAS, 'frA', nVASI, 'in', 1);
  opt = addLink(opt, nVASI, 'out', nVASQ, 'in', 0);
  opt = addLink(opt, nVASQ, 'out', nSR, 'bk', 1);

  % for POX AR
  opt = addLink(opt, nPOAS,    'bkA', nPOIXARi, 'frA', 1);
  opt = addLink(opt, nPOIXARi, 'frA', nVIXARiI, 'in', 1);
  opt = addLink(opt, nPOIXARi, 'bkA', nPOIXARo, 'frA', 1);
  opt = addLink(opt, nPOIXARo, 'frA', nVIXARoI, 'in', 1);
  opt = addLink(opt, nVIXARiI, 'out', nVIXARiQ, 'in', 0);
  opt = addLink(opt, nVIXARiQ, 'out', nBSPX,    'frB', 1);
  opt = addLink(opt, nVIXARoI, 'out', nVIXARoQ, 'in', 0);
  opt = addLink(opt, nVIXARoQ, 'out', nBSPX, 'bkA', 1);

  % for POY AR
  opt = addLink(opt, nPOIXARo, 'bkA', nPOIYARi, 'frA', 1);
  opt = addLink(opt, nPOIYARi, 'frA', nVIYARiI, 'in', 1);
  opt = addLink(opt, nPOIYARi, 'bkA', nPOIYARo, 'frA', 1);
  opt = addLink(opt, nPOIYARo, 'frA', nVIYARoI, 'in', 1);
  opt = addLink(opt, nVIYARiI, 'out', nVIYARiQ, 'in', 0);
  opt = addLink(opt, nVIYARiQ, 'out', nBSPY, 'frB', 1);
  opt = addLink(opt, nVIYARoI, 'out', nVIYARoQ, 'in', 0);
  opt = addLink(opt, nVIYARoQ, 'out', nBSPY, 'bkA', 1);

  % to REFL PDs
  opt = addLink(opt, nPOIYARo, 'bkA', nPOAttREFL, 'frA', 1);
  opt = addLink(opt, nPOAttREFL, 'frA', nVAttREFLI, 'in', 1);
  opt = addLink(opt, nVAttREFLI, 'out', nVAttREFLQ, 'in', 0);
  opt = addLink(opt, nVAttREFLQ, 'out', nAttREFL, 'bkA', 1);

  % to AS PD
  opt = addLink(opt, nPOAttREFL, 'bkA', nPOAttAS, 'frA', 1);
  opt = addLink(opt, nPOAttAS, 'frA', nVAttASI, 'in', 1);
  opt = addLink(opt, nVAttASI, 'out', nVAttASQ, 'in', 0);
  opt = addLink(opt, nVAttASQ, 'out', nAttAS, 'bkA', 1);

  % to OMCR PD
  opt = addLink(opt, nPOAttAS, 'bkA', nPOAttOMC, 'frA', 1);
  opt = addLink(opt, nPOAttOMC, 'frA', nVAttOMCI, 'in', 1);
  opt = addLink(opt, nVAttOMCI, 'out', nVAttOMCQ, 'in', 0);
  opt = addLink(opt, nVAttOMCQ, 'out', nAttOMC, 'bkA', 1);
  
  % to POX PD
  opt = addLink(opt, nPOAttOMC, 'bkA', nPOAttPOX, 'frA', 1);
  opt = addLink(opt, nPOAttPOX, 'frA', nVAttPOXI, 'in', 1);
  opt = addLink(opt, nVAttPOXI, 'out', nVAttPOXQ, 'in', 0);
  opt = addLink(opt, nVAttPOXQ, 'out', nAttPOX, 'bkA', 1);
  %opt = addLink(opt, nPOAttPOX, 'bkA', nPOAtt???, 'frA', 1);

  
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % demodulation phases

  % Phases must be set 0 for the first time running. Please write phase
  % names you want to set here. Actual phases are set automatically in
  % optXXXX_demph.m. Run loopnoise once and press 'y' for the first time 
  % when demod phase re-configuration is asked, then run loopnoise again.

  phAS1 = 0;
  phASM = 0;
  phASP = 0;
  
  phREFL1 = 0;
  phREFL2 = 0;
  phREFLM = 0;
  phREFLP = 0;
  phREFLb1 = 0;
  phREFLbM = 0;
  phREFLbP = 0;
  phREFLb2 = 0;
  phREFL3f1 = 0;
  phREFL3f2 = 0;
  phREFL7f1 = 0;
  
  phOMC1 = 0;
  phOMC2 = 0;
  phOMCM = 0;
  phOMCP = 0;
  phOMCRb1 = 0;
  phOMCRb2 = 0;
  phOMCRbM = 0;
  phOMCRbP = 0;
  phOMCRb3f1 = 0; 
  phOMCRb3f2 = 0;
  phOMCRb7f1 = 0; 
  
  phPOX1 = 0;
  phPOXM = 0;
  phPOXP = 0;
  phPOX2 = 0;
  phPOX3f1 = 0;
  phPOX3f2 = 0;
  phPOX7f1 = 0;
  
  phPOXb1 = 0;
  phPOXbM = 0;
  phPOXbP = 0;
  phPOXb2 = 0;
  
  phPOY1 = 0;
  phPOYM = 0;
  phPOYP = 0;
  phPOY2 = 0;
  
  phPOM1 = 0;
  phPOMM = 0;
  phPOMP = 0;
  phPOM2 = 0;
  
  %%% load demodulation phases if dem phases are updated
  if exist(strcat(mfilename, '_demph.m'), 'file') > 0
    eval(strcat(mfilename, '_demph'))
  end

  
  
    
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Add Probes

  % double-demod sum (plus) and difference (minus) frequencies
  fp = f2 + f1;
  fm = f2 - f1;
  
  % REFL signals (reflected or symmetric port)
  opt = addProbeAt(opt, 'REFL DC', nREFL, 'in',  0, 0);             % DC
  opt = addProbeAt(opt, 'REFL I1', nREFL, 'in', f1, phREFL1);       % f1 demod I
  opt = addProbeAt(opt, 'REFL Q1', nREFL, 'in', f1, phREFL1 + 90);  % f1 demod Q
  opt = addProbeAt(opt, 'REFL IM', nREFL, 'in', fm, phREFLM);       % fm demod I
  opt = addProbeAt(opt, 'REFL QM', nREFL, 'in', fm, phREFLM + 90);  % fm demod Q
  opt = addProbeAt(opt, 'REFL IP', nREFL, 'in', fp, phREFLP);       % fp demod I
  opt = addProbeAt(opt, 'REFL QP', nREFL, 'in', fp, phREFLP + 90);  % fp demod Q
  opt = addProbeAt(opt, 'REFL I2', nREFL, 'in', f2, phREFL2);       % f2 demod I
  opt = addProbeAt(opt, 'REFL Q2', nREFL, 'in', f2, phREFL2 + 90);  % f2 demod Q
  opt = addProbeAt(opt, 'REFL I3f1', nREFL, 'in', f1 * 3, phREFL3f1);       % 3 f1 demod I
  opt = addProbeAt(opt, 'REFL Q3f1', nREFL, 'in', f1 * 3, phREFL3f1 + 90);  % 3 f1 demod Q
  opt = addProbeAt(opt, 'REFL I3f2', nREFL, 'in', f2 * 3, phREFL3f2);       % 3 f2 demod I
  opt = addProbeAt(opt, 'REFL Q3f2', nREFL, 'in', f2 * 3, phREFL3f2 + 90);  % 3 f2 demod Q
  opt = addProbeAt(opt, 'REFL I7f1', nREFL, 'in', f1 * 7, phREFL7f1);       % 7 f1 demod I
  opt = addProbeAt(opt, 'REFL Q7f1', nREFL, 'in', f1 * 7, phREFL7f1 + 90);  % 7 f1 demod Q
  %opt = addProbeAt(opt, 'REFL If2M2f1', nREFL, 'in', f2 - f1 * 2, phREFLf2M2f1);       % 3 f1 demod I
  %opt = addProbeAt(opt, 'REFL Qf2M2f1', nREFL, 'in', f2 - f1 * 2, phREFLf2M2f1 + 90);  % 3 f1 demod Q

  opt = addProbeAt(opt, 'REFLb DC', nREFLb, 'in',  0, 0);               % DC
  opt = addProbeAt(opt, 'REFLb I1', nREFLb, 'in', f1, phREFLb1);               % f1 demod I
  opt = addProbeAt(opt, 'REFLb Q1', nREFLb, 'in', f1, phREFLb1 + 90);              % f1 demod Q
  opt = addProbeAt(opt, 'REFLb IM', nREFLb, 'in', fm, phREFLbM);        % fm demod I
  opt = addProbeAt(opt, 'REFLb QM', nREFLb, 'in', fm, phREFLbM + 90);   % fm demod Q
  opt = addProbeAt(opt, 'REFLb IP', nREFLb, 'in', fp, phREFLbP);        % fp demod I
  opt = addProbeAt(opt, 'REFLb QP', nREFLb, 'in', fp, phREFLbP + 90);	% fp demod Q
  opt = addProbeAt(opt, 'REFLb I2', nREFLb, 'in', f2, phREFLb2);       % f2 demod I
  opt = addProbeAt(opt, 'REFLb Q2', nREFLb, 'in', f2, phREFLb2 + 90);  % f2 demod Q

  % AS signals (anti-symmetric port)
  opt = addProbeAt(opt, 'AS DC', nAS, 'in',  0, 0);             % DC
  opt = addProbeAt(opt, 'AS I1', nAS, 'in', f1, phAS1);         % f1 demod I
  opt = addProbeAt(opt, 'AS Q1', nAS, 'in', f1, phAS1 + 90);    % f1 demod Q
  opt = addProbeAt(opt, 'AS IM', nAS, 'in', fm, phASM);         % fm demod I
  opt = addProbeAt(opt, 'AS QM', nAS, 'in', fm, phASM + 90);    % fm demod Q
  opt = addProbeAt(opt, 'AS IP', nAS, 'in', fp, phASP);         % fp demod I
  opt = addProbeAt(opt, 'AS QP', nAS, 'in', fp, phASP + 90);    % fp demod Q

  % POX signals (IX pick-off)
  opt = addProbeAt(opt, 'POX DC', nPOX, 'in',  0, 0);		    % DC
  opt = addProbeAt(opt, 'POX I1', nPOX, 'in', f1, phPOX1);		% f1 demod I
  opt = addProbeAt(opt, 'POX Q1', nPOX, 'in', f1, phPOX1 + 90);	% f1 demod Q
  opt = addProbeAt(opt, 'POX IM', nPOX, 'in', fm, phPOXM);		% fm demod I
  opt = addProbeAt(opt, 'POX QM', nPOX, 'in', fm, phPOXM + 90);	% fm demod Q
  opt = addProbeAt(opt, 'POX IP', nPOX, 'in', fp, phPOXP);		% fp demod I
  opt = addProbeAt(opt, 'POX QP', nPOX, 'in', fp, phPOXP + 90);	% fp demod Q
  opt = addProbeAt(opt, 'POX I2', nPOX, 'in', f2, phPOX2);		% f2 demod I
  opt = addProbeAt(opt, 'POX Q2', nPOX, 'in', f2, phPOX2 + 90);	% f2 demod Q
  opt = addProbeAt(opt, 'POX I3f1', nPOX, 'in', 3 * f1, phPOX3f1);		% 3 f1 demod I
  opt = addProbeAt(opt, 'POX Q3f1', nPOX, 'in', 3 * f1, phPOX3f1 + 90);	% 3 f1 demod Q
  opt = addProbeAt(opt, 'POX I3f2', nPOX, 'in', 3 * f2, phPOX3f2);		% 3 f2 demod I
  opt = addProbeAt(opt, 'POX Q3f2', nPOX, 'in', 3 * f2, phPOX3f2 + 90);	% 3 f2 demod Q
  opt = addProbeAt(opt, 'POX I7f1', nPOX, 'in', 7 * f1, phPOX7f1);		% 7 f1 demod I
  opt = addProbeAt(opt, 'POX Q7f1', nPOX, 'in', 7 * f1, phPOX7f1 + 90);	% 7 f1 demod Q
  %opt = addProbeAt(opt, 'POX If2M2f1', nPOX, 'in', f2 - f1 * 2, phPOXf2M2f1);		% 3 f1 demod I
  %opt = addProbeAt(opt, 'POX Qf2M2f12', nPOX, 'in', f2 - f1 * 2, phPOXf2M2f1 + 90);	% 3 f1 demod Q

  opt = addProbeAt(opt, 'POXb DC', nPOXb, 'in',  0, 0);        % DC
  opt = addProbeAt(opt, 'POXb I1', nPOXb, 'in', f1, phPOXb1);        % f1 demod I
  opt = addProbeAt(opt, 'POXb Q1', nPOXb, 'in', f1, phPOXb1 + 90);        % f1 demod Q
  opt = addProbeAt(opt, 'POXb IM', nPOXb, 'in', fm, phPOXbM);		% fm demod I
  opt = addProbeAt(opt, 'POXb QM', nPOXb, 'in', fm, phPOXbM + 90);	% fm demod Q
  opt = addProbeAt(opt, 'POXb IP', nPOXb, 'in', fp, phPOXbP);		% fp demod I
  opt = addProbeAt(opt, 'POXb QP', nPOXb, 'in', fp, phPOXbP + 90);	% fp demod Q
  opt = addProbeAt(opt, 'POXb I2', nPOXb, 'in', f2, phPOXb2);		% f2 demod I
  opt = addProbeAt(opt, 'POXb Q2', nPOXb, 'in', f2, phPOXb2 + 90);	% f2 demod Q

  % POY signals (IY pick-off)
  opt = addProbeAt(opt, 'POY DC', nPOY, 'in',  0,  0);         % DC
  opt = addProbeAt(opt, 'POY I1', nPOY, 'in', f1,  phPOY1);         % f1 demod I
  opt = addProbeAt(opt, 'POY Q1', nPOY, 'in', f1,  phPOY1 + 90);         % f1 demod Q
  opt = addProbeAt(opt, 'POY IM', nPOY, 'in', fm, phPOYM);		% fm demod I
  opt = addProbeAt(opt, 'POY QM', nPOY, 'in', fm, phPOYM + 90);	% fm demod Q
  opt = addProbeAt(opt, 'POY IP', nPOY, 'in', fp, phPOYP);		% fp demod I
  opt = addProbeAt(opt, 'POY QP', nPOY, 'in', fp, phPOYP + 90);	% fp demod Q
  opt = addProbeAt(opt, 'POY I2', nPOY, 'in', f2, phPOY2);		% f2 demod I
  opt = addProbeAt(opt, 'POY Q2', nPOY, 'in', f2, phPOY2 + 90);	% f2 demod Q

  % POM signals (PRM pick-off placed between PR and BS)
  opt = addProbeAt(opt, 'POM DC', nPOM, 'in',  0, 0);		    % DC
  opt = addProbeAt(opt, 'POM I1', nPOM, 'in', f1, phPOM1);		% f1 demod I
  opt = addProbeAt(opt, 'POM Q1', nPOM, 'in', f1, phPOM1 + 90);	% f1 demod Q
  opt = addProbeAt(opt, 'POM IM', nPOM, 'in', fm, phPOMM);		% fm demod I
  opt = addProbeAt(opt, 'POM QM', nPOM, 'in', fm, phPOMM + 90);	% fm demod Q
  opt = addProbeAt(opt, 'POM IP', nPOM, 'in', fp, phPOMP);		% fp demod I
  opt = addProbeAt(opt, 'POM QP', nPOM, 'in', fp, phPOMP + 90);	% fp demod Q
  opt = addProbeAt(opt, 'POM I2', nPOM, 'in', f2, phPOM2);		% f2 demod I
  opt = addProbeAt(opt, 'POM Q2', nPOM, 'in', f2, phPOM2 + 90);	% f2 demod Q

  % OMC reflected signals
  opt = addProbeAt(opt, 'OMC DC', nOMC, 'in',  0,  0);          % DC
  opt = addProbeAt(opt, 'OMC I1', nOMC, 'in', f1, phOMC1);     % f1 demod I
  opt = addProbeAt(opt, 'OMC Q1', nOMC, 'in', f1, phOMC1 + 90);% f1 demod Q
  opt = addProbeAt(opt, 'OMC IM', nOMC, 'in', fm, phOMCM);		% fm demod I
  opt = addProbeAt(opt, 'OMC QM', nOMC, 'in', fm, phOMCM + 90); % fm demod Q
  opt = addProbeAt(opt, 'OMC IP', nOMC, 'in', fp, phOMCP);		% fp demod I
  opt = addProbeAt(opt, 'OMC QP', nOMC, 'in', fp, phOMCP + 90);	% fp demod Q
  opt = addProbeAt(opt, 'OMC I2', nOMC, 'in', f2, phOMC2);      % f2 demod Q
  opt = addProbeAt(opt, 'OMC Q2', nOMC, 'in', f2, phOMC2 + 90); % f2 demod Q

  % OMCRb reflected signals
  opt = addProbeAt(opt, 'OMCRb DC', nOMCRb, 'in',  0, 0);		% DC
  opt = addProbeAt(opt, 'OMCRb I1', nOMCRb, 'in', f1, phOMCRb1);		% f1 demod I
  opt = addProbeAt(opt, 'OMCRb Q1', nOMCRb, 'in', f1, phOMCRb1 + 90);	% f1 demod Q
  opt = addProbeAt(opt, 'OMCRb IM', nOMCRb, 'in', fm, phOMCRbM);		% fm demod I
  opt = addProbeAt(opt, 'OMCRb QM', nOMCRb, 'in', fm, phOMCRbM + 90);	% fm demod Q
  opt = addProbeAt(opt, 'OMCRb IP', nOMCRb, 'in', fp, phOMCRbP);		% fp demod I
  opt = addProbeAt(opt, 'OMCRb QP', nOMCRb, 'in', fp, phOMCRbP + 90);	% fp demod Q
  opt = addProbeAt(opt, 'OMCRb I2', nOMCRb, 'in', f2, phOMCRb2);		% f2 demod Q
  opt = addProbeAt(opt, 'OMCRb Q2', nOMCRb, 'in', f2, phOMCRb2 + 90);	% f2 demod Q
  opt = addProbeAt(opt, 'OMCRb I3f1', nOMCRb, 'in', 3 * f1, phOMCRb3f1);		% 3 f1 demod Q
  opt = addProbeAt(opt, 'OMCRb Q3f1', nOMCRb, 'in', 3 * f1, phOMCRb3f1 + 90);	% 3 f1 demod Q
  opt = addProbeAt(opt, 'OMCRb I3f2', nOMCRb, 'in', 3 * f2, phOMCRb3f2);		% 3 f2 demod Q
  opt = addProbeAt(opt, 'OMCRb Q3f2', nOMCRb, 'in', 3 * f2, phOMCRb3f2 + 90);	% 3 f2 demod Q
  opt = addProbeAt(opt, 'OMCRb I7f1', nOMCRb, 'in', 7 * f1, phOMCRb7f1);		% 7 f1 demod Q
  opt = addProbeAt(opt, 'OMCRb Q7f1', nOMCRb, 'in', 7 * f1, phOMCRb7f1 + 90);	% 7 f1 demod Q
  %opt = addProbeAt(opt, 'OMCRb If2M2f12', nOMCRb, 'in', f2 - f1 * 2, phOMCRbf2M2f1);		% 3 f1 demod Q
  %opt = addProbeAt(opt, 'OMCRb Qf2M2f12', nOMCRb, 'in', f2 - f1 * 2, phOMCRbf2M2f1 + 90);	% 3 f1 demod Q

  % Homodyne signals
  opt = addProbeAt(opt, 'HDa DC', nHDa, 'in',  0,  0);          % DC
  opt = addProbeAt(opt, 'HDb DC', nHDb, 'in',  0,  0);          % DC

  
  % transmitted DC signals
  opt = addProbeAt(opt, 'TRX DC', nTRX, 'in', 0, 0);            % DC
  opt = addProbeAt(opt, 'TRY DC', nTRY, 'in', 0, 0);            % DC
  
  % temporary probe
  opt = addProbeAt(opt, 'TMP', nPR, 'bk', 0, 0);
